"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.help = exports.commandNames = exports.shorthands = exports.cliOptionsTypes = exports.rcOptionsTypes = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const logger_1 = require("@pnpm/logger");
const os_env_path_extender_1 = require("@pnpm/os.env.path-extender");
const render_help_1 = __importDefault(require("render-help"));
const rcOptionsTypes = () => ({});
exports.rcOptionsTypes = rcOptionsTypes;
const cliOptionsTypes = () => ({
    force: Boolean,
});
exports.cliOptionsTypes = cliOptionsTypes;
exports.shorthands = {};
exports.commandNames = ['setup'];
function help() {
    return (0, render_help_1.default)({
        description: 'Sets up pnpm',
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    {
                        description: 'Override the PNPM_HOME env variable in case it already exists',
                        name: '--force',
                        shortAlias: '-f',
                    },
                ],
            },
        ],
        url: (0, cli_utils_1.docsUrl)('setup'),
        usages: ['pnpm setup'],
    });
}
exports.help = help;
function getExecPath() {
    // @ts-expect-error
    if (process['pkg'] != null) {
        // If the pnpm CLI was bundled by vercel/pkg then we cannot use the js path for npm_execpath
        // because in that case the js is in a virtual filesystem inside the executor.
        // Instead, we use the path to the exe file.
        return process.execPath;
    }
    return (require.main != null) ? require.main.filename : process.cwd();
}
function copyCli(currentLocation, targetDir) {
    const newExecPath = path_1.default.join(targetDir, path_1.default.basename(currentLocation));
    if (path_1.default.relative(newExecPath, currentLocation) === '')
        return;
    logger_1.logger.info({
        message: `Copying pnpm CLI from ${currentLocation} to ${newExecPath}`,
        prefix: process.cwd(),
    });
    fs_1.default.mkdirSync(targetDir, { recursive: true });
    fs_1.default.copyFileSync(currentLocation, newExecPath);
}
async function handler(opts) {
    const execPath = getExecPath();
    if (execPath.match(/\.[cm]?js$/) == null) {
        copyCli(execPath, opts.pnpmHomeDir);
    }
    try {
        const report = await (0, os_env_path_extender_1.addDirToEnvPath)(opts.pnpmHomeDir, {
            configSectionName: 'pnpm',
            proxyVarName: 'PNPM_HOME',
            overwrite: opts.force,
            position: 'start',
        });
        return renderSetupOutput(report);
    }
    catch (err) { // eslint-disable-line
        switch (err.code) {
            case 'ERR_PNPM_BAD_ENV_FOUND':
                err.hint = 'If you want to override the existing env variable, use the --force option';
                break;
            case 'ERR_PNPM_BAD_SHELL_SECTION':
                err.hint = 'If you want to override the existing configuration section, use the --force option';
                break;
        }
        throw err;
    }
}
exports.handler = handler;
function renderSetupOutput(report) {
    if (report.oldSettings === report.newSettings) {
        return 'No changes to the environment were made. Everything is already up to date.';
    }
    const output = [];
    if (report.configFile) {
        output.push(reportConfigChange(report.configFile));
    }
    output.push(`Next configuration changes were made:
${report.newSettings}`);
    if (report.configFile == null) {
        output.push('Setup complete. Open a new terminal to start using pnpm.');
    }
    else if (report.configFile.changeType !== 'skipped') {
        output.push(`To start using pnpm, run:
source ${report.configFile.path}
`);
    }
    return output.join('\n\n');
}
function reportConfigChange(configReport) {
    switch (configReport.changeType) {
        case 'created': return `Created ${configReport.path}`;
        case 'appended': return `Appended new lines to ${configReport.path}`;
        case 'modified': return `Replaced configuration in ${configReport.path}`;
        case 'skipped': return `Configuration already up to date in ${configReport.path}`;
    }
}
//# sourceMappingURL=setup.js.map